//! \addtogroup fefrontend
//! \{
//!
#ifndef _FeatExtractorFF_h
#define _FeatExtractorFF_h

#include<vector>
#include<math.h>

// local
#include "DynamicDataBuffer.h"

#define PI    3.14159265358979323846
#define TWOPI 6.28318530717958647692
//#define ACCUM_LOG_FB_MEAN // turn on if the logFB mean should be accumulated

class MelFBank ; // here just declared, defined in FeatExtractorFF.cpp



//! Feature extractor

//! Description
//!
class FeatExtractorFF {
 public:
    FeatExtractorFF(int sampFr=8000, bool isOnln=false); // construtor
    ~FeatExtractorFF() ;                  // destructor

    void setInput(void *in) { feInput = (vector<short> *) in ; }
    void *getInput() { return (void *)feInput ; }
    const void *getOutput() { return (void *)&feOutput; }
    void reset() ;
    void init() ;
    void run() ;
		//void OfflineRun();

	  short getParamKind() { return paramKind ; }
    void setOnline() { isOnline = true ; }
    void circFrameDCOffPreem(int numSamples) ;
    void circFrame2wFrameWithWin() ;
    void procWinFrame() ;
    void four1 (float *data, short nn, short isign) ;
    void realft (float *data, float * backdata,short n, short isign) ;
//void realft (float *data, short n, short isign) ;
    void powerSpec() ;
    void freqFilter(char kind = 'A') ;
    void logEnergy() ;
    void shortTimeEnergy();
    void logFbMeanRemoval() ;
    void loadLogFbMean() ;
    void lowHighFreqFeat() ;
    int  getLowFreqFeatIdx() ;
    int  getHiFreqFeatIdx() ;
#ifdef ACCUM_LOG_FB_MEAN
    void printAverLogFbMean() ;
#endif
		void zeroCrossingRate();
    int sign(float );
    void subBandLogEnergies();
    void spectralCentroid();
    void spectralBandwidth();
 	  int TrnCos( int Inv, int OrdAna, int NumCof, float *Ent, float *Sal) ;
	
 protected:
    vector<short>* feInput ;
    FeatureBuffer feOutput ; // change to a cleverer buffer
    FeatureBuffer fbOutput ; 
 private:

	long tmpcount;

	void logFbMeanCalc() ;
	void utAcumLogFbMean() ;
	void utLogFbMeanCalc() ;
	void utLogFbVarCalc() ;
	void utLogFbMeanRemoval() ;
	bool doUtLogFbMeanRem ;

	long frCounter ;

    // related to feInput
    //unsigned short feInputReadPos, feInputCanRead ;
    unsigned long feInputReadPos, feInputCanRead ; //para buffers con mayor longiutd es necesario que sea long
    
    int sampFreq ;
    bool isOnline ;

    // circFrame, wFrame
    float* circFrame ;
    float* wFrame;                 // windowed frame
    float* coefFrame;                 // windowed frame
	  float* absFftFrame;
    unsigned short frSize, frShift ;
    unsigned short fftSize, halfFftSize ;
    unsigned short cfReadPos, cfWritePos ;

    // DC offset removal and preemphasis
    float prevDCInSamp, prevDCOutSamp ;
    float prevPreemInSamp, preemCoef ;
    
    // mel FB analysis
    unsigned short fbSize ;
    float fbFloor ;
    int melStartFreq ;
    bool melNorm ;
    MelFBank *melFBextor ;  //melbank 16
	  MelFBank *melFBspec;  //melbank 20
	  int kind_ff ;
    // logFB mean removal
    char* fnLogFbMean ;
    char* defFnLogFbMean ;
    bool doLogFbMeanRem ;
    float* logFbMean ;
		float* logFbVar ;
#ifdef ACCUM_LOG_FB_MEAN
    double* accumLogFbMean ;
    
#endif
bool remLoHiChan;
bool remLoHiFFcoef;
	bool justEn;
    bool isFirstFrame ;
    bool FF0_DCT1 ; //0 is FF, 1 is DCT 
    float alpha ;
    bool addEnergy ; //1 append frame energy to cepstral vector  
    bool addlogEnergy ; //1 append log energy to cepstal vector
    bool powSqrt ; //1 computes sqrt of powerSpectrum 
    float  *win ;
    float  powSpecFloor ;
    float  logEnFloor ;
    bool   calcLowFreqFeat, calcHiFreqFeat;
		bool calcZcr, calcSpecCen, calcSpecBand, calcSbLogEn;
    float logEn ;
    float shortTimeEn;
    double /*logEn,*/ lowFreqFeat, hiFreqFeat ;
    int    logEnIdx, EnIdx, lffIdx, hffIdx , zcrIdx, specCenIdx, specBandIdx, sbLogEnIdx;
    float zeroCrossRate;
    float* subBLEnergies; 
    int numSubBands, numBandCoefs;
    float weightSpecSum,specSum, specCentroid;
    float specBandwidth;
	  float coefZERO ;
    unsigned short coeffSize ;    
    unsigned short ceplifter ; //0 means not liftering, >0 applies liftering od order ceplifter
    bool zero_coeff ;
    short paramKind;
    bool DCoffset ;
		bool doVNorm;

		//cuda variables
		 float* cu_wFrame;   
int memsize;
} ;


//! \}
//!

#endif // _FeatExtractorFF_h

