#define _ParamTable_

// c libraries

// c++ libraries
#include<cstdio>
#include<cstddef>
#include<list>

// local libraries
#include "ParamTable.h"

/* -- start of global stuff -- */
#define MAX_LINE_LENGTH 256
#define TXT_HOLDER_SIZE 2048 // was 1024 ; DM March 19 2006
char  txtHolder[TXT_HOLDER_SIZE] ;
char* freeTxt = &txtHolder[0] ;
unsigned long freeTxtLength = TXT_HOLDER_SIZE ;
char* insertToTxtHolder(const char *s)
{   // insert ascii text and return the pointer
    unsigned int len = strlen(s) + 1 ;
    if(len > freeTxtLength) {
    printf ("Parameter file long; increase TXT_HOLDER_SIZE in ParamTable.cpp (currently %i). Exitig.", TXT_HOLDER_SIZE) ;
    exit(1) ;
    }
    char *tmp = freeTxt ;
    strcpy(freeTxt, s) ;
    freeTxt += len ;
    freeTxtLength -= len ;
    return tmp ;
}


ParamTable* globalParTab = NULL ; //!< Global parameter table

//provides access to the global parameter table
ParamTable* globalParamTable()
{   
    if(globalParTab == NULL) {//if does not exist
    globalParTab = new ParamTable ;
    globalParTab->scopeName = insertToTxtHolder("null") ;
    }
    return globalParTab ;//returns a pointer to the table
}
/* -- end of global stuff -- */

/* -- start of utility functions -- */
char* skipSpace (char *p)
{
    char c = *p ;
    while( c==' ' || c=='\t' || c=='\n' || c=='\014' ) {
    p++ ; c = *p ;
    }
    if (c=='\0') return NULL ;
    else         return p ;
}

char* readSymbol(char* p, char* sym)
{ // copy symbol from p to sym ; return poiter to p after the read symbol
    char c ;
    p = skipSpace(p) ;
    if(p == NULL) return NULL ;
    while((c=*p) != '\0') {
    if( c==' ' || c=='\t' || c=='\n' || c=='\014' || c=='=' )
        break ;
    else {
        *sym++ = c ;
        p++ ;
    }
    }
    *sym++ = '\0' ;
    return p ;
}

int strcmpNC(const char* s, const char* t)
{ // case insensitive strcomp
    for( ; tolower(*s) == tolower(*t) ; ++s, ++t)
    if(*s == '\0') return 0 ;
    return tolower(*s) - tolower(*t) ;
}

int entrySortComp(const void* e1, const void* e2) {
    return strcmpNC( ((ParamEntry*) e1)->name, ((ParamEntry*) e2)->name ) ;
}

int scopeSortComp(const void* ss1, const void* ss2) {
  ParamTable* s1 = *((ParamTable**) ss1) ;
  ParamTable* s2 = *((ParamTable**) ss2) ;
  return strcmpNC(s1->scopeName, s2->scopeName) ;
}
/* -- end of utility functions -- */

/* -- start of constructor/destructor -- */
ParamEntry::ParamEntry() : name(NULL), kind(iType), parent(NULL) {;}  // constructor
ParamTable::ParamTable() : parent(NULL) {;} // constructor
ParamTable::~ParamTable() // destructor
{
    for(size_t i=0 ; i<children.size() ; ++i) delete (ParamTable*) children[i] ;
}
/* -- end of constructor/destructor -- */

void
ParamTable::load(char* fn)
{
    FILE* f ;
    f = fopen(fn, "r") ;
    if(f == NULL) {
    printf("ParamTable::load error openning file '%s'\n", fn) ;
    exit(1) ;
    }
    readScopeRecur(f) ;
    fclose(f) ;
    sortForSearch() ;
}

void
ParamTable::readScopeRecur(FILE* f)
{
    char line   [MAX_LINE_LENGTH] ;
    char auxMem [MAX_LINE_LENGTH << 1] ;
    char auxMemX[] = "0x00000000" ; // hex 32 bits = long
    ParamEntry auxParEntry ;

    while(fgets(line, MAX_LINE_LENGTH, f) != NULL) {
    char* p = line ;
    p = readSymbol(p, auxMem) ;  // returns NULL when empty line
    if(p == NULL) continue ; // empty line

    if((auxMem[0] == '<') && (auxMem[1] != '/')) { // make new scope <scope_name>
        auxMem[strlen(auxMem) - 1] = '\0' ;        // replace '>' by '\0'
        children.push_back((void*) new ParamTable) ;
        ((ParamTable*) children.back())->scopeName = insertToTxtHolder(auxMem+1) ;
        ((ParamTable*) children.back())->parent    = this ;
        ((ParamTable*) children.back())->readScopeRecur(f) ;
    }
    else if((auxMem[0] == '<') && (auxMem[1] == '/')) { // if equals end of the scope </scope_name>
        break ;
    }
    else { // read param entries
        entries.push_back(auxParEntry) ;
        ParamEntry* pe = & entries.back() ;
        pe->name       = insertToTxtHolder(auxMem) ;
        pe->parent     = this ;

        if((p = skipSpace(p)) == NULL || *p != '=')
        printf("ParamTable:load, equal sign failure at line: '%s'", line) ;

        ++p ; // skip '='

        if((p = readSymbol(p, auxMem)) == NULL)
        printf("ParamTable:load, value failure at line: '%s'", line) ;

        if( isdigit(auxMem[0]) || ((auxMem[0] == '-') && (isdigit(auxMem[1]))) ) {
        if(auxMem[0]=='0' && auxMem[1]=='x') { // hexadecimal 0x...
            int strLength = strlen(auxMem) ;
            for(int i=0 ; i<4 ; i++) pe->val.ul[i] = 0x0 ; // init to 0
            for(int i=0 ; i<4 ; i++) { // 4 long elements
            if(strLength < 2) break ;
            strcpy(auxMemX, "0x00000000") ; // hex 32 bits = long
            for(int j=0 ; j<8 ; j++) { // each long element is 8 hex chars
                if(--strLength < 2) break ;
                auxMemX[9-j] = auxMem[strLength] ;
            }
            sscanf(auxMemX, "%lx", &(pe->val.ul[i]));
            }
            pe->kind = ulType ;
        }
        else {
            int i=0;
            while(auxMem[i] != '\0' && auxMem[i] != '.') {
            ++i;
            }
            if(auxMem[i] == '.') { // float
            pe->kind = fType ;
            sscanf(auxMem, "%g", &(pe->val.f));
            }
            else {                 // int
            pe->kind = iType;
            pe->val.i = atoi(auxMem);
            }
        }
        }
        else {                         // string
        pe->kind = sType ;
        pe->val.s = insertToTxtHolder(auxMem) ;
        }
    }
    }
    return ;
} // End readScopeRecur

void
ParamTable::sortForSearch()
{ // sort according to entry name and scopename (make searchable)
  if(entries.size() > 1)
    qsort( &entries[0], entries.size(), sizeof(ParamEntry), entrySortComp ) ;

  if(children.size() > 0)
      qsort( &children[0], children.size(), sizeof(children[0]), scopeSortComp ) ;
  for(size_t i=0 ; i<children.size() ; i++)
      ((ParamTable*) children[i])->sortForSearch() ;
}

ParamEntry*
ParamTable::search(char* name)
{ // search only local scope,
  ParamEntry tmp ;
  tmp.kind = sType ;
  tmp.name = name ;
  ParamEntry* pe ;
  pe = (ParamEntry*) bsearch ((void*) &tmp, (void*) &entries[0], entries.size(), sizeof(ParamEntry), entrySortComp) ;
  return pe ;
}

ParamTable*
ParamTable::searchScope(char *name)
{
  ParamTable tmp ;
  tmp.scopeName = name ;
  ParamTable* ptmp = &tmp ;
  ParamTable** p ;

  p = (ParamTable**) bsearch ((void*) &ptmp, &children[0], children.size(), sizeof(children[0]), scopeSortComp) ;
  if(p == NULL)
      return NULL ;
  return *p ;
}

void
ParamTable::print(FILE *f)
{
    fprintf(f, "<") ;
    if(scopeName == NULL)
    fprintf(f, "null") ;
    else
    fprintf (f, "%s", scopeName) ;

    if(parent == NULL)
    fprintf(f, " [root]") ;
    fprintf(f, ">\n") ;
    for(size_t i=0 ; i<entries.size() ; i++) {
    ParamEntry pe = entries[i] ;
    fprintf(f, "%s=", pe.name) ;
    switch(pe.kind) {
        case iType:  fprintf(f, "%d\n",   pe.val.i) ;  break ;
        case fType:  fprintf(f, "%.2e\n", pe.val.f) ;  break ;
        case sType:  fprintf(f, "%s\n",   pe.val.s) ;  break ;
        case ulType: fprintf(f, "%ld\n",  pe.val.ul[0]) ; break ;
    }
    }
    for(size_t i=0 ; i<children.size() ; i++) {
    ParamTable* p = (ParamTable*) children[i] ;
    p->print(f) ;
    }
    fprintf (f, "</") ;
    if(scopeName == NULL)
    fprintf(f, "null") ;
    else
    fprintf(f, "%s", scopeName) ;
    if(parent == NULL) fprintf(f, " [root]") ;
    fprintf(f, ">\n") ;

} // end print
