//! \addtogroup fefrontend
//! \{
//!
#ifndef _ParamTable_h
#define _ParamTable_h

#include <vector>

using namespace std;

typedef class ParamTable ParamTable ;
typedef class ParamEntry ParamEntry ;

// Parameter entry
//   type
typedef enum { sType , iType , fType , ulType } paramType ;//!< Param types
//   value
//! Structure that stores diferent kind of values for any param
typedef union {
    char* s ;  //!< string param
    int   i ;  //!< int param
    float f ;  //!< float param
    unsigned long ul[4] ; //!< for reading long bit masks (asures 128 bits)
} paramValue ;


//! Class that defines a param entrie
class ParamEntry { // all public class
 public:
    ParamEntry() ;    //!< Constructor
    ~ParamEntry() {;} //!< Destructor

    char*       name ;
    paramType   kind ;
    paramValue  val ;
    ParamTable* parent ; //!< the parent scope
} ;


//! Class for easy read of a parameter file. 

//! How to integrate the class in your code:
//!  
//! 	- Get a pointer to the global parameter table:
//!	
//!	\htmlonly  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;   \endhtmlonly 
//!	<CODE> ParamTable* parTab = globalParamTable() ; </CODE>
//!
//! 	- Load a file 
//!	
//!	\htmlonly  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;   \endhtmlonly 
//!   <CODE> parTab->load("your_file") ; </CODE>
//! - A typical form to get a single param entry, could be:
//!	
//!	\htmlonly  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;   \endhtmlonly 
//! 	<CODE>ParamEntry* pe ; 
//!	\htmlonly  <BR>  &nbsp;&nbsp;&nbsp;&nbsp;  \endhtmlonly 
//!		string paramvalue ; //suppose the value is a string
//!	\htmlonly  <BR>   &nbsp;&nbsp;&nbsp;&nbsp;  \endhtmlonly 
//! 		if((pe = params->search("your_param")) != NULL) paramvalue = pe->val.s ;
//!	\htmlonly  <BR>  &nbsp;&nbsp;&nbsp;&nbsp;  \endhtmlonly 
//! 		else fnRootSVMModel = defFnRootSVMModel ; </CODE>
//!
class ParamTable {
 public:
    ParamTable() ;  //!< Constructor
    ~ParamTable() ; //!< Destructor

    void load(char* fn) ; //!< Loads paramTable from file
    void sortForSearch() ;	
    ParamEntry* search(char* name) ;
    ParamTable* searchScope(char* name) ;
    void readScopeRecur(FILE* f) ;
    void print(FILE* f) ;	//!< Prints a config file

    char*         scopeName ;
    ParamTable*   parent ;   //parent is surely existing
    vector<void*> children ; //but children not

 private:
    vector<ParamEntry> entries ;//!< Vector of file entries
} ;

ParamTable* globalParamTable() ; //!< Provides access to the global parameter table. Note that the global condition of the param table, alows any parameter to be available at any point of the code

#endif // _ParamTable_h

//! \}
//!
