//! \addtogroup fefrontend
//! \{
//!
// c libraries

// c++ libraries
#include<fstream>
#include <iostream>
#include <sstream>
#include<string>
#include<vector>

// local libraries
#include "FeatExtractorFF.h"
#include "ParamTable.h"
#include <sys/time.h>


//----------------------------------------------
class InOut{
public:
	InOut() {;} // constructor
	~InOut() { free(defaultDir) ; } // destructor

	void init() ;
	
	int STAT0_DA1;
	int sampFreq ;
	int inChunkSize ;
	char* defaultDir ;
	char* defaultFileList ;
} ;
//! \}
//
//----------------------------------------------
void
InOut::init()
{
	
	ParamTable* ioParams = (globalParamTable())->searchScope("InOut") ;
	ParamEntry* pe ;
	if(ioParams != NULL) {
		if((pe = ioParams->search("sampFreq")) != NULL) sampFreq = pe->val.i ;
		else sampFreq = 8000 ;
		if((pe = ioParams->search("inChunkSize")) != NULL) inChunkSize = pe->val.i ;
		else inChunkSize = 1024 ; //1024 
		if((pe = ioParams->search("STAT0_DA1")) != NULL) STAT0_DA1 =  pe->val.i ;
		else STAT0_DA1 = 0 ;
	}
	else { // all default
		sampFreq     = 8000 ;
		inChunkSize  = 1024 ;
		STAT0_DA1 = 0;
	}
}
//----------------------------------------------
//! \addtogroup fefrontend
//! \{
//!
//! \file feature-extraction.cpp

// //! \file AEDDetectionTest.cpp
// //! \brief An offline implementation for acoustic event detection propose
//! hola adew
//! The input signal is assumed to be in a raw format using the 16-bit sample representation with Little Endian (Intel) byte ordering. The input sampling frequency, as well as other parameters can configured by the user through a configuration file. 
//! The ouput feature extraction file is written in HTK format 
//! See ref aedSec2 to know how to run it.
//! Note that if the parameter file is not provided the following default values are used:
//! - fnFileList = "./list"
//! - inputDir = "./" 
//! - outputDir = "./" 
//! - featDir = "./" 
//! - inputSuf = empty
//! - outputSuf = empty
//! - featSuf = empty
//! - saveFeat = 0
//! - sampFreq = 16000
//! - inChunkSize = 1024
//! 
//! No inputRef* parameters needed.
//!
int 
main (int argc, char* argv[])
{
	timeval tim;
	gettimeofday(&tim, NULL);
	double t1=tim.tv_sec+(tim.tv_usec/1000000.0);



		
  	InOut ioParms ;
	
	
	//Read params
	if(argc < 4){
		cout << "feat-extraction::ERROR: too few parameters used " << argc << endl;
		exit(0);
	}
	ParamTable* parTab = globalParamTable() ;
  parTab->load(argv[1]) ;
  ioParms.init() ;
	
	vector<short> inChunk(ioParms.inChunkSize);
	FeatExtractorFF* featExtor = new FeatExtractorFF (ioParms.sampFreq, false) ;
		
  featExtor->setInput(&inChunk) ;
	FeatureBuffer* ffFeatures = (FeatureBuffer *) featExtor->getOutput() ;

	ostringstream inFile, outFile ;
	inFile << argv[2] ;outFile << argv[3] ;

	//<cout << "feat-extraction: Processing input file " << inFile.str().c_str() << endl ;
	ifstream inputStream (inFile.str().c_str());	
	if (!inputStream) {   cout << "feat-extraction::ERROR: can't open input file " << inFile.str().c_str()<< endl; exit(0); }

	gettimeofday(&tim, NULL);
	double t2=tim.tv_sec+(tim.tv_usec/1000000.0);
	printf("%.6lf seconds elapsed\n", t2-t1);
	//Feature extraction 
	featExtor->reset() ;
	while ( !inputStream.eof() ) {
		inputStream.read((char *)&(inChunk[0]), sizeof(short) * ioParms.inChunkSize) ;	
		featExtor->run() ;
	}
	inputStream.close();
	gettimeofday(&tim, NULL);
	double t3=tim.tv_sec+(tim.tv_usec/1000000.0);
	printf("%.6lf seconds elapsed\n", t3-t1);
	//Write feature file 
	if(ioParms.STAT0_DA1==0){
		ffFeatures->saveSelFFtoHTK(outFile.str().c_str(),false);
	}else if(ioParms.STAT0_DA1==1){
		ffFeatures->saveSelFFtoHTK(outFile.str().c_str(),true);
	}else{
		cout << "feat-extraction::ERROR: No STAT nor DA feature file output selected" << endl ;
	}
	//
	//cout << "feat-extraction: End of feature extraction" << inFile.str().c_str() << endl ;
	/*gettimeofday(&tim, NULL);
	double t4=tim.tv_sec+(tim.tv_usec/1000000.0);
	printf("%.6lf seconds elapsed\n", t4-t1);*/
}

//! \}



